;===================================================================
;
; PROGRAM:  mollosvd_trans
; 
; PURPOSE:  post-processing (smoothing and down-grading) of
;           the "super-channel" maps generated by mollosvd_make
; INPUT  :  raw channel maps
; OUTPUT :  a "parameter file" and a fits file containing the data.
;           Fits files are channel map files and can be directly
;           viewed with ds9. 
; USAGE  :  This is a multi-step process
;           (1) use mollosvd_make to generate "super-channel" maps,
;           i.e.high-resolution ppv cubes. These are the raw data and
;           are generally at higher spectral (and spatial) resolution
;           than observed data sets.
;           filename: mollosvd{axis}_{emitter}_{dump#}.fits
;           filename: mollosvd{axis}_{emitter}_{dump#}.head
;           (2) these ppv-cubes can be degraded to "observations",
;           using mollosvd_trans[late]. Degrading in velocity space
;           is done by binning, degrading in position space uses 
;           a Gaussian filter.
;           filename: mollosvd{axis}_{emitter}_{dump#}_c{channel#}s{beamwidth}e{noise}.fits
;           filename: mollosvd{axis}_{emitter}_{dump#}_c{channel#}s{beamwidth}e{noise}.head
;           The beamwidth is in pixels, and the noise level is in mJy.
; NOTES   : ilo:ihi and jlo:jhi area ALWAYS in the plane of sky.
;           Most of the RT part is from Rybicki & Lightman
;           Assumes Rayleigh-Jeans limit, i.e converts to brightness temperature
;--------------------------------------------------------------------

pro mollosvd_trans,itest=itest

  descr = 'm3c0a0a_r1'

  if (descr eq 'Gf2') then begin; core right of center
    legend0= '!6Gf2'
    models = 1
    mdl    = '3aaww00d0d1_g'
    dmp    = [18]
    dtdmp  = 8e-3
    idir   = 1
    whichmol= [0]
    nbins    = 60l
    beamwidth   = [0] ; in pixel
    noise   = 0.0 ; in Kelvin
    disk     = '~/proteus/run/collimflow/'
    odisk    = '~/proteus/run/collimflow/plots/'
    specdisk = '~/proteus/run/collimflow/tables/'
  endif
  
  if (descr eq 'c3f0g3b') then begin; core right of center
    legend0= '!6c3f0g3b'
    models = 3
    mdl    = 'c3f0g3b_1'
    dmp    = [25,25,25]
    dtdmp  = 2.16e-3
    idir   = 2
    angle  = [45,60,75]
    whichmol= [6]
    nbins    = 80l
    beamwidth   = [2] ; in pixel
    noise   = 0.01 ; in Kelvin
    disk     = '~/proteus/run/hvc/'
    odisk    = '~/proteus/run/hvc/plots/'
    specdisk = '~/proteus/run/hvc/tables/'
  endif

  if (descr eq 'm3c0a0a_r1') then begin; core right of center
    legend0= '!6m3c0a0a_r1'
    models = 2
    mdl    = 'm3c0a0a_r1'
    dmp    = [32,32,32]
    dtdmp  = 2.16e-3
    idir   = 2
    angle  = [60,75]
    whichmol= [6]
    nbins    = 80l
    beamwidth   = [2] ; in pixel
    noise   = 0.0 ; in Kelvin
    disk     = '~/proteus/run/hvc/'
    odisk    = '~/proteus/run/hvc/plots/'
    specdisk = '~/proteus/run/hvc/tables/'
  endif

  if (descr eq 'testrot') then begin; core right of center
    legend0= '!6testrot'
    models = 2
    mdl    = 'testrot'
    dmp    = [0,0,0]
    dtdmp  = 2.16e-3
    idir   = 2
    angle  = [45,75]
    whichmol= [6]
    nbins    = 50l
    beamwidth   = [1] ; in pixel
    noise   = 0.02 ; in Kelvin
    disk     = '~/proteus/run/hvc/'
    odisk    = '~/proteus/run/hvc/plots/'
    specdisk = '~/proteus/run/hvc/tables/'
  endif
  
;===========================================================================

; the molecule "database"
  nallmol     = 6
  nmol        = n_elements(whichmol)
  struct_12c16o_10 = {MOL, fillab:'12C16O-10'}
  struct_12c16o_21 = {MOL, fillab:'12C16O-21'}
  struct_12c16o_32 = {MOL, fillab:'12C16O-32'}
  struct_13c16o_10 = {MOL, fillab:'13C16O-10'}
  struct_13c16o_21 = {MOL, fillab:'13C16O-21'}
  struct_13c16o_32 = {MOL, fillab:'13C16O-32'}
  struct_h21cm     = {MOL, fillab:'HI-21cm'}
;  struct_13co= {MOL, mwgt:29.0, nu0:1.102e11, gu:2, gl:1, a_ul:6.3e-8, ncr:1.9e3, abnd:5e-8, fillab:'13CO'}
;  struct_c18o= {MOL, mwgt:30.0, nu0:1.098e11, gu:2, gl:1, a_ul:6.3e-8, ncr:1.9e3, abnd:3e-6, fillab:'C18O'}
;  struct_nh3 = {MOL, mwgt:17.0, nu0:2.300e10, gu:1, gl:1, a_ul:1.7e-7, ncr:1.8e3, abnd:1e-8, fillab:'NH3'}
;  struct_hco = {MOL, mwgt:29.0, nu0:8.920e10, gu:2, gl:1, a_ul:3.0e-5, ncr:1.7e5, abnd:1e-8, fillab:'HCOp'}
;  struct_cs  = {MOL, mwgt:44.0, nu0:9.800e10, gu:4, gl:2, a_ul:1.7e-5, ncr:3.0e5, abnd:1e-8, fillab:'CS'}
;  struct_h2co= {MOL, mwgt:40.0, nu0:1.408e11, gu:2, gl:1, a_ul:5.4e-5, ncr:1.1e6, abnd:1e-8, fillab:'H2CO'}
;  struct_hcn = {MOL, mwgt:27.0, nu0:8.860e10, gu:2, gl:1, a_ul:2.4e-5, ncr:2.6e6, abnd:1e-8, fillab:'HCN'}

  molcat    = replicate(struct_12c16o_10,nallmol+1)
  molcat[1] = struct_12c16o_21
  molcat[2] = struct_12c16o_32
  molcat[3] = struct_13c16o_10
  molcat[4] = struct_13c16o_21
  molcat[5] = struct_13c16o_32
  molcat[6] = struct_h21cm

  !PATH = !PATH + ":~/lib/astrolib"

  fitsheadrd = strarr(11)
  fitsheadwr = strarr(13)

;=====================================================================
; constants
  pi     = 4d0*atan(1d0)
  clight = 3.0d10
  kboltz = 1.381d-16
  hplanck= 6.626d-27
  
  nx     = 0l
  ny     = 0l
  nv     = 0l
  nbeam  = n_elements(beamwidth)

  mintbr = 0.0
  maxtbr = 0.0

  iseed  = 151l

  if (n_elements(angle) gt 0) then isangle=1 else isangle=0

  for m=0,models-1 do begin
    numstr = string(dmp[m],format='(i4.4)')
    if (isangle eq 1) then astr = '_a'+string(angle[m],format='(i3.3)')+'_' else astr = ''
    for im=0,nmol-1 do begin
      tmol = molcat[whichmol[im]]
      fnm = disk+mdl+'/mollosvd'+astr+string(idir,format='(i1)')+'_'+tmol.fillab+'_'+numstr+'.fits'
      print,'MOLLOSVD_TRANS: reading file ',fnm
      rawcube = ptr_new(mrdfits(fnm,0,fitsheadrd))
      help,fitsheadrd
      print,"MOLLOSVD: fits header:"
      print,fitsheadrd
      reads,fitsheadrd[3],nx,    format='(11x,i20)'
      reads,fitsheadrd[4],ny,    format='(11x,i20)'
      reads,fitsheadrd[5],nv,    format='(11x,i20)'
      reads,fitsheadrd[6],minvph,format='(11x,e20.5)'
      reads,fitsheadrd[7],maxvph,format='(11x,e20.5)'
      reads,fitsheadrd[8],mintbr,format='(11x,e20.5)'
      reads,fitsheadrd[9],maxtbr,format='(11x,e20.5)'
      print,format='("MOLLOSVD_TRANS: MOLECU = ",a10)',tmol.fillab
      print,format='("MOLLOSVD_TRANS: NAXIS1 = ",i5)',nx
      print,format='("MOLLOSVD_TRANS: NAXIS2 = ",i5)',ny
      print,format='("MOLLOSVD_TRANS: NAXIS3 = ",i5)',nv
      print,format='("MOLLOSVD_TRANS: MINVLS = ",e13.5)',minvph
      print,format='("MOLLOSVD_TRANS: MAXVLS = ",e13.5)',maxvph
      print,format='("MOLLOSVD_TRANS: MINTBR = ",e13.5)',mintbr
      print,format='("MOLLOSVD_TRANS: MAXTBR = ",e13.5)',maxtbr

      print,format='("MOLLOSVD_TRANS: min/max(rawcube) = ",e13.5,e13.5)',min(*rawcube),max(*rawcube)

;     rebin the channels. The rebinned channels are multiplied by dv
;     -> K km s^{-1}
      dv = (maxvph-minvph)/float(nv)
      redcube = ptr_new(fltarr(nx,ny,nbins))
      for ic = 0,nbins-1 do begin
        for ic1 = 0,nv/nbins-1 do begin
          (*redcube)[*,*,ic] = (*redcube)[*,*,ic] + (*rawcube)[*,*,ic*nv/nbins+ic1]*dv
        endfor
      endfor
      rgrid    = ptr_new(fltarr(nx,ny))
      for iy=-ny/2,ny/2-1 do begin
        for ix=-nx/2,nx/2-1 do begin
          (*rgrid)[ix+nx/2,iy+ny/2] = float(ix)^2 + float(iy)^2
        endfor
      endfor
      for ib = 0,nbeam-1 do begin
        sredcube = ptr_new(fltarr(nx,ny,nbins))
        tlayer   = ptr_new(fltarr(nx,ny))
        if (beamwidth[ib] gt 0) then begin
          gaussian = ptr_new(0.5*exp(-0.5*(*rgrid)[*,*]/float(beamwidth[ib])^2)/(pi*float(beamwidth[ib])^2))
          sigcut = min([2*beamwidth[ib]^2,nx/2])
;         we have non-periodic domains, so we need to restrict the
;         smoothing filter 
          for iy=0,ny-1 do begin
            jlo = max([0,iy-sigcut])
            jhi = min([ny-1,iy+sigcut-1])
            llo = max([ny/2-sigcut,ny/2-iy])
            lhi = min([ny/2+sigcut-1,ny/2+ny-iy-1])
            for ix=0,nx-1 do begin
              ilo    = max([0,ix-sigcut])
              ihi    = min([nx-1,ix+sigcut-1])
              klo    = max([nx/2-sigcut,nx/2-ix])
              khi    = min([nx/2+sigcut-1,nx/2+nx-ix-1])
              sgauss = total((*gaussian)[klo:khi,llo:lhi])
              for ic = 0,nbins-1 do begin
                (*tlayer)[*,*] = reform((*redcube)[*,*,ic])
                (*sredcube)[ix,iy,ic] = total((*tlayer)[ilo:ihi,jlo:jhi]*(*gaussian)[klo:khi,llo:lhi])/sgauss
              endfor
            endfor
          endfor
          ptr_free,tlayer,gaussian
        endif else begin
          (*sredcube)[*,*,*] = (*redcube)[*,*,*]
        endelse
;       add noise
        for ic = 0,nbins-1 do begin  
          noisearr = ptr_new(randomn(iseed,nx,ny))
          (*sredcube)[*,*,ic] = (*sredcube)[*,*,ic] + noise*dv*(*noisearr)
          ptr_free,noisearr
        endfor
        fnm = disk+mdl+'/mollosvd'+astr+string(idir,format='(i1)')+'_'$
                  +tmol.fillab+'_'+numstr+'c'+string(nbins,format='(i3.3)')+'s'$
                  +string(beamwidth[ib],format='(i2.2)')+'e'+string(fix(noise*100.0),format='(i3.3)')+'.fits'
        print,'MOLLOSVD_TRANS: writing file ',fnm
        fitsheadwr = strarr(9)
        fitsheadwr[0] = string(nx,                            format='("NAXIS1  = ",i19  ," / [pix]")')
        fitsheadwr[1] = string(ny,                            format='("NAXIS2  = ",i19  ," / [pix]")')
        fitsheadwr[2] = string(nbins,                         format='("NAXIS3  = ",i19  ," / [chn]")')
        fitsheadwr[3] = string(minvph,                        format='("MINVLS  = ",e20.5," / [km/s]")')
        fitsheadwr[4] = string(maxvph,                        format='("MAXVLS  = ",e20.5," / [km/s]")')
        fitsheadwr[5] = string(min((*sredcube)),              format='("MINTBR  = ",e20.5," / [K km/s]")')
        fitsheadwr[6] = string(max((*sredcube)),              format='("MAXTBR  = ",e20.5," / [K km/s]")')
        fitsheadwr[7] = string(beamwidth[ib]                , format='("BEAMWD  = ",i20  ," / [pix]")')
        mwrfits,(*sredcube),fnm,fitsheadwr,/CREATE
        fitsheadwr = 0
        ptr_free,sredcube
      endfor
      ptr_free,rawcube,redcube,rgrid       
    endfor
  endfor
 
;===============================================================

  return
end

